/*
    ES Knot
    Copyright 2009 - 2010 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#include "st_martin_esknot_KnotRenderer.h"
#ifdef USE_OES_2
#include "knot2.h"
#else
#include "knot.h"
#endif
#include "jniutils.h"
#include <dlfcn.h>

#define getKnotPrivData getPrivData<Knot*>
#define setKnotPrivData setPrivData<Knot*>

JavaVM* vm;

void debug(const char* str) {
	JNIEnv* env = NULL;
	vm->GetEnv((void**) &env, JNI_VERSION_1_4);
	debugPrintf(env, "%s", str);
}

void* customGetProcAddress(const char* name) {
	return dlsym(RTLD_DEFAULT, name);
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_nativeOnDrawFrame(JNIEnv* env, jobject thiz, jlong time) {
	Knot* knot = getKnotPrivData(env, thiz);
	if (!knot) {
		glClearColor(0, 0, 0, 1);
		glClear(GL_COLOR_BUFFER_BIT);
		return;
	}
	knot->draw(time);
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_nativeOnSurfaceCreated(JNIEnv* env, jobject thiz) {
	Knot* knot = getKnotPrivData(env, thiz);
	delete knot;
	setKnotPrivData(env, thiz, NULL);
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_nativeOnSurfaceChanged(JNIEnv* env, jobject thiz, jint width, jint height) {
	env->GetJavaVM(&vm);
	Knot* knot = getKnotPrivData(env, thiz);
	if (knot) {
		glViewport(0, 0, width, height);
		knot->setSize(width, height);
	} else {
		knot = new Knot(width, height);
		knot->setTexMode(Knot::TEXSPEC);
		knot->setUpdateVertices(true);

		debugPrintf(env, "vendor: %s", glGetString(GL_VENDOR));
		debugPrintf(env, "renderer: %s", glGetString(GL_RENDERER));
		debugPrintf(env, "version: %s", glGetString(GL_VERSION));
		debugPrintf(env, "extensions: %s", glGetString(GL_EXTENSIONS));
	}
	setKnotPrivData(env, thiz, knot);
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_close(JNIEnv* env, jobject thiz) {
	Knot* knot = getKnotPrivData(env, thiz);
	delete knot;
	setKnotPrivData(env, thiz, NULL);
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_enableRotate(JNIEnv* env, jobject thiz, jboolean rotate) {
	Knot* knot = getKnotPrivData(env, thiz);
	if (!knot)
		return;
	knot->enableRotate(rotate);
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_rotate(JNIEnv* env, jobject thiz, jint x, jint y) {
	Knot* knot = getKnotPrivData(env, thiz);
	if (!knot)
		return;
	knot->rotate(x, y);
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_startMorph(JNIEnv* env, jobject thiz) {
	Knot* knot = getKnotPrivData(env, thiz);
	if (!knot)
		return;
	knot->startMorph();
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_enableGravity(JNIEnv* env, jobject thiz, jboolean gravity) {
	Knot* knot = getKnotPrivData(env, thiz);
	if (!knot)
		return;
	knot->enableGravity(gravity);
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_setGravity(JNIEnv* env, jobject thiz, jfloat x, jfloat y, jfloat z) {
	Knot* knot = getKnotPrivData(env, thiz);
	if (!knot)
		return;
	knot->setGravity(x, y, z);
}

JNIEXPORT jboolean JNICALL Java_st_martin_esknot_KnotRenderer_getMatrixSupported(JNIEnv* env, jobject thiz) {
	Knot* knot = getKnotPrivData(env, thiz);
	if (!knot)
		return false;
	return knot->getMatrixSupported();
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_setTexMode(JNIEnv* env, jobject thiz, jint mode) {
	Knot* knot = getKnotPrivData(env, thiz);
	if (!knot)
		return;
	switch (mode) {
	case 0:
		knot->setTexMode(Knot::UNTEXTURED);
		break;
	case 1:
		knot->setTexMode(Knot::PLAINTEX);
		break;
	case 2:
		knot->setTexMode(Knot::TEXSPEC);
		break;
	case 3:
		knot->setTexMode(Knot::CELSHADED);
		break;
	}
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_setUpdateVertices(JNIEnv* env, jobject thiz, jboolean update) {
	Knot* knot = getKnotPrivData(env, thiz);
	if (!knot)
		return;
	knot->setUpdateVertices(update);
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_setRadialBlur(JNIEnv* env, jobject thiz, jint mode) {
	Knot* knot = getKnotPrivData(env, thiz);
	if (!knot)
		return;
	switch (mode) {
	case 0:
		knot->setRadialBlur(false);
		knot->setRadialGlow(false);
		break;
	case 1:
		knot->setRadialBlur(true);
		knot->setRadialGlow(false);
		break;
	case 2:
		knot->setRadialBlur(true);
		knot->setRadialGlow(true);
		break;
	}
}

JNIEXPORT void JNICALL Java_st_martin_esknot_KnotRenderer_setMotionBlur(JNIEnv* env, jobject thiz, jint mode) {
	Knot* knot = getKnotPrivData(env, thiz);
	if (!knot)
		return;
	knot->setMotionBlur(mode ? true : false);
}






