/*
    ES Knot
    Copyright 2006 - 2010 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#include <SDL.h>
#ifdef USE_OES_2
#include <GLES2/gl2.h>
#include "knot2.h"
#else
#include <GLES/gl.h>
#include "knot.h"
#endif
#ifdef SDL_EGL
#include <EGL/egl.h>
#include <SDL_syswm.h>
#endif
#include <stdio.h>
#include <stdlib.h>

#ifdef SCREENSHOT
#define WIDTH 512
void makeScreenShot();
#define HEIGHT 512
#else
#define WIDTH 176
#define HEIGHT 208
#endif

#ifdef SDL_EGL
EGLDisplay display;
EGLSurface surface;
EGLContext context;

const EGLint attribList[] = {
	EGL_DEPTH_SIZE, 16,
	EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
#ifdef USE_OES_2
	EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
#endif
	EGL_NONE
};

bool initEGL() {
	SDL_SysWMinfo wminfo;
	SDL_VERSION(&wminfo.version)
	SDL_GetWMInfo(&wminfo);
	SDL_ShowCursor(0);

	display = eglGetDisplay((EGLNativeDisplayType) wminfo.info.x11.display);
	if (display == EGL_NO_DISPLAY) {
		fprintf(stderr, "eglGetDisplay failed, %d\n", eglGetError());
		return false;
	}
	if (eglInitialize(display, NULL, NULL) == EGL_FALSE) {
		fprintf(stderr, "eglInitialize failed, %d\n", eglGetError());
		return false;
	}
	eglBindAPI(EGL_OPENGL_ES_API);

	EGLConfig config;
	EGLint numConfigs;
	if (eglChooseConfig(display, attribList, &config, 1, &numConfigs) == EGL_FALSE) {
		fprintf(stderr, "eglChooseConfig failed, %d\n", eglGetError());
		return false;
	}

#ifdef USE_OES_2
	const EGLint ctxattr[] = { EGL_CONTEXT_CLIENT_VERSION, 2, EGL_NONE };
	const EGLint* attrs = ctxattr;
#else
	const EGLint* attrs = NULL;
#endif
	context = eglCreateContext(display, config, NULL, attrs);
	if (!context) {
		fprintf(stderr, "eglCreateContext failed, %d\n", eglGetError());
		return false;
	}
	surface = eglCreateWindowSurface(display, config, (void*) wminfo.info.x11.window, NULL);
	if (!surface) {
		fprintf(stderr, "eglCreateWindowSurface failed, %d\n", eglGetError());
		return false;
	}

	if (eglMakeCurrent(display, surface, surface, context) == EGL_FALSE) {
		fprintf(stderr, "eglMakeCurrent failed, %d\n", eglGetError());
		return false;
	}

	printf("EGL vendor: %s\n", eglQueryString(display, EGL_VENDOR));
	printf("EGL version: %s\n", eglQueryString(display, EGL_VERSION));
	printf("EGL extensions: %s\n", eglQueryString(display, EGL_EXTENSIONS));

	return true;
}

void closeEGL() {
	eglMakeCurrent(display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	eglDestroySurface(display, surface);
	eglDestroyContext(display, context);
	eglTerminate(display);
	eglReleaseThread();
}

void swapBuffers() {
	eglSwapBuffers(display, surface);
}
#else
void swapBuffers() {
	SDL_GL_SwapBuffers();
}

extern "C" GL_API int GL_APIENTRY _dgles_load_library(void *, void *(*)(void *, const char *));

#endif



void debug(const char *str) {
	printf("%s\n", str);
}

static void *proc_loader(void *h, const char *name) {
	return SDL_GL_GetProcAddress(name);
}

int main(int argc, char *argv[]) {
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE) < 0) {
		fprintf(stderr, "SDL error: %s\n", SDL_GetError());
		return 1;
	}
	atexit(SDL_Quit);

#ifndef SDL_EGL
	int err;
#if defined(WIN32)
	err = SDL_GL_LoadLibrary("opengl32.dll");
#elif defined(__linux__) || defined(__FreeBSD__)
	err = SDL_GL_LoadLibrary("libGL.so");
#elif defined(__APPLE__)
	err = SDL_GL_LoadLibrary("/usr/X11R6/lib/libGL.dylib");
#else
#error Your platform is not supported
	err = 1;
#endif
	if (err != 0) {
		fprintf(stderr, "Can't load OpenGL library: %s\n", SDL_GetError());
		return 1;
	}

	SDL_GL_SetAttribute(SDL_GL_ACCUM_RED_SIZE, 0);
	SDL_GL_SetAttribute(SDL_GL_ACCUM_GREEN_SIZE, 0);
	SDL_GL_SetAttribute(SDL_GL_ACCUM_BLUE_SIZE, 0);
	SDL_GL_SetAttribute(SDL_GL_ACCUM_ALPHA_SIZE, 0);

	int flags = SDL_OPENGL | SDL_RESIZABLE;
	int width = WIDTH;
	int height = HEIGHT;

#else
	int flags = SDL_FULLSCREEN;
	int width = 0, height = 0;
#endif
	SDL_WM_SetCaption("ESKnot", NULL);
	SDL_Surface* screen = SDL_SetVideoMode(width, height, 0, flags);
	if (screen == NULL) {
		fprintf(stderr, "%s\n", SDL_GetError());
		return 1;
	}

#ifdef SDL_EGL
	if (!initEGL())
		return 1;
#else
	_dgles_load_library(NULL, proc_loader);
#endif

	printf("vendor: %s\n", glGetString(GL_VENDOR));
	printf("renderer: %s\n", glGetString(GL_RENDERER));
	printf("version: %s\n", glGetString(GL_VERSION));
	printf("extensions: %s\n", glGetString(GL_EXTENSIONS));

	Knot *k = new Knot(screen->w, screen->h);
	bool update = true;
	bool glow = false;
	bool radial = false;
	bool blur = false;
	Knot::TexMode mode = Knot::TEXSPEC;
	k->setTexMode(mode);
	k->setUpdateVertices(update);
#ifdef SCREENSHOT
	k->enableRotate(false);
#endif

	Uint32 starttime = SDL_GetTicks();

	int distance = 0;
	while (1) {
		SDL_Event event;
		bool done = false;
		while (SDL_PollEvent(&event)) {
			if (event.type == SDL_QUIT) {
				done = true;
			} else if (event.type == SDL_KEYDOWN) {
				switch (event.key.keysym.sym) {
				case SDLK_ESCAPE:
				case SDLK_BACKSPACE:
				case SDLK_q:
					done = true;
					break;
				case SDLK_SPACE:
					k->startMorph();
					break;
				case SDLK_1:
					mode = Knot::UNTEXTURED;
					k->setTexMode(mode);
					break;
				case SDLK_2:
					mode = Knot::PLAINTEX;
					k->setTexMode(mode);
					break;
				case SDLK_3:
					mode = Knot::TEXSPEC;
					k->setTexMode(mode);
					break;
				case SDLK_4:
					mode = Knot::CELSHADED;
					k->setTexMode(mode);
					break;
				case SDLK_u:
					update = !update;
					k->setUpdateVertices(update);
					break;
				case SDLK_r:
					radial = !radial;
					glow = false;
					k->setRadialBlur(radial);
					k->setRadialGlow(glow);
					break;
				case SDLK_g:
					glow = !glow;
					radial = glow;
					k->setRadialBlur(radial);
					k->setRadialGlow(glow);
					break;
				case SDLK_b:
					blur = !blur;
					k->setMotionBlur(blur);
					break;
				case SDLK_t:
					mode = (Knot::TexMode) ((mode + 1) % Knot::MAX_TEXMODE);
					k->setTexMode(mode);
					break;
				default:
					break;
				}
			} else if (event.type == SDL_VIDEORESIZE) {
				if (SDL_SetVideoMode(event.resize.w, event.resize.h, 0, SDL_OPENGL | SDL_RESIZABLE) == NULL) {
					fprintf(stderr, "%s\n", SDL_GetError());
					return 1;
				}
				k->setSize(event.resize.w, event.resize.h);
				k->reinit();
			} else if (event.type == SDL_MOUSEBUTTONDOWN) {
				k->enableRotate(false);
				distance = 0;
			} else if (event.type == SDL_MOUSEBUTTONUP) {
				if (distance < 10)
					k->startMorph();
				k->enableRotate(true);
			} else if (event.type == SDL_MOUSEMOTION) {
				if (event.motion.state & SDL_BUTTON(1))
					k->rotate(event.motion.xrel, event.motion.yrel);
				distance += abs(event.motion.xrel) + abs(event.motion.yrel);
			}
		}
		if (done) break;

		k->draw(SDL_GetTicks() - starttime);
#ifdef SCREENSHOT
		makeScreenShot();
#endif

//		glFlush();
//		glFinish();
		swapBuffers();
		SDL_Delay(1);
	}

	// Using FBO on GLES2 on SGX on N900 may hang if cleaning up properly
	// too fast. This delay is enough for making it work...
	SDL_Delay(200);

	delete k;

#ifdef SDL_EGL
	closeEGL();
#endif
	return 0;
}

